From: Thomas Gleixner <tglx@linutronix.de>
To: Sudeep Holla <sudeep.holla@arm.com>
Cc: LKML <linux-kernel@vger.kernel.org>,
Ingo Molnar <mingo@kernel.org>,
Peter Zijlstra <peterz@infradead.org>,
Preeti U Murthy <preeti@linux.vnet.ibm.com>,
Suzuki Poulose <Suzuki.Poulose@arm.com>,
Lorenzo Pieralisi <Lorenzo.Pieralisi@arm.com>,
Catalin Marinas <Catalin.Marinas@arm.com>,
"Rafael J. Wysocki" <rafael.j.wysocki@intel.com>
Subject: Re: [patch 1/2] tick/broadcast: Prevent deep idle states if no broadcast device available
Date: Mon, 6 Jul 2015 18:06:03 +0200 (CEST) [thread overview]
Message-ID: <alpine.DEB.2.11.1507061750170.3916@nanos> (raw)
In-Reply-To: <559AA256.5040305@arm.com>
On Mon, 6 Jul 2015, Sudeep Holla wrote:
> On 06/07/15 16:35, Thomas Gleixner wrote:
> > Well, we should figure out what happens while we are at it before
> > everything gets paged out again.
> >
>
> True. I just wanted to mention that this patch works for all the
> practical purposes.
>
> > In the case of CONFIG_NOHZ=n and CONFIG_HIGHRES=n the broadcast
> > hrtimer is not compiled as it depends on CONFIG_TICK_ONESHOT, so it
> > works via the bc.evtdev == NULL check.
> >
> > With either option enabled CONFIG_TICK_ONESHOT gets set, so the
> > broadcast timer gets installed but somehow does not work proper if
> > nohz and highres are disabled on the kernel command line.
> >
>
> Let me know if you want to test something to help debug this configuration.
Can you try the following delta patch?
Thanks,
tglx
---
Index: tip/kernel/time/tick-broadcast.c
===================================================================
--- tip.orig/kernel/time/tick-broadcast.c
+++ tip/kernel/time/tick-broadcast.c
@@ -360,14 +360,15 @@ void tick_broadcast_control(enum tick_br
cpumask_set_cpu(cpu, tick_broadcast_on);
if (!cpumask_test_and_set_cpu(cpu, tick_broadcast_mask)) {
/*
* Only shutdown the cpu local device, if the
* broadcast device exists and is in periodic
* mode. The latter check prevents a hickup
* during the switch from periodic to oneshot
* mode.
*/
- if (bc != NULL && tick_broadcast_device.mode ==
- TICKDEV_MODE_PERIODIC)
+ if (bc && !(bc->features & CLOCK_EVT_FEAT_HRTIMER) &&
+ tick_broadcast_device.mode == TICKDEV_MODE_PERIODIC)
clockevents_shutdown(dev);
}
break;
@@ -697,14 +698,18 @@ int __tick_broadcast_oneshot_control(enu
* shutdown.
*/
ret = broadcast_needs_cpu(bc, cpu);
+ if (ret)
+ goto out;
/*
- * If the hrtimer broadcast check tells us that the
- * cpu cannot go deep idle, or if the broadcast device
- * is in periodic mode, we just return.
+ * If the broadcast device is in periodic mode, we
+ * return.
*/
- if (ret || tick_broadcast_device.mode == TICKDEV_MODE_PERIODIC)
- goto out;
+ if (tick_broadcast_device.mode == TICKDEV_MODE_PERIODIC) {
+ /* If it is a hrtimer based broadcast, return busy */
+ if (bc->features & CLOCK_EVT_FEAT_HRTIMER)
+ ret = -EBUSY;
+ }
if (!cpumask_test_and_set_cpu(cpu, tick_broadcast_oneshot_mask)) {
WARN_ON_ONCE(cpumask_test_cpu(cpu, tick_broadcast_pending_mask));
next prev parent reply other threads:[~2015-07-06 16:06 UTC|newest]
Thread overview: 25+ messages / expand[flat|nested] mbox.gz Atom feed top
2015-07-05 20:53 [patch 0/2] tic/broadcast: Plug a few corner cases which cause malfunction Thomas Gleixner
2015-07-05 20:53 ` [patch 1/2] tick/broadcast: Prevent deep idle states if no broadcast device available Thomas Gleixner
2015-07-06 15:09 ` Sudeep Holla
2015-07-06 15:35 ` Thomas Gleixner
2015-07-06 15:44 ` Sudeep Holla
2015-07-06 16:06 ` Thomas Gleixner [this message]
2015-07-06 16:27 ` Sudeep Holla
2015-07-06 16:53 ` Thomas Gleixner
2015-07-06 17:57 ` Sudeep Holla
2015-07-06 19:56 ` Thomas Gleixner
2015-07-07 7:31 ` Thomas Gleixner
2015-07-07 11:25 ` Sudeep Holla
2015-07-07 11:55 ` Thomas Gleixner
2015-07-07 17:12 ` [tip:timers/urgent] tick/broadcast: Prevent hrtimer recursion tip-bot for Thomas Gleixner
2015-07-07 17:12 ` [tip:timers/urgent] tick/broadcast: Sanity check the shutdown of the local clock_event tip-bot for Thomas Gleixner
2015-07-07 17:13 ` [tip:timers/urgent] tick/broadcast: Make idle check independent from mode and config tip-bot for Thomas Gleixner
2015-07-07 17:13 ` [tip:timers/urgent] tick/broadcast: Prevent deep idle if no broadcast device available tip-bot for Thomas Gleixner
2015-07-07 17:13 ` [tip:timers/urgent] tick/broadcast: Move the check for periodic mode inside state handling tip-bot for Thomas Gleixner
2015-07-07 17:14 ` [tip:timers/urgent] tick/broadcast: Return busy if periodic mode and hrtimer broadcast tip-bot for Thomas Gleixner
2015-07-07 17:14 ` [tip:timers/urgent] tick/broadcast: Return busy when IPI is pending tip-bot for Thomas Gleixner
2015-07-07 17:14 ` [tip:timers/urgent] tick/broadcast: Check for hrtimer broadcast active early tip-bot for Thomas Gleixner
2015-07-05 20:53 ` [patch 2/2] tick/broadcast: Handle spurious interrupts gracefully Thomas Gleixner
2015-07-06 15:17 ` Sudeep Holla
2015-07-06 15:36 ` Thomas Gleixner
2015-07-07 17:15 ` [tip:timers/urgent] " tip-bot for Thomas Gleixner
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=alpine.DEB.2.11.1507061750170.3916@nanos \
--to=tglx@linutronix.de \
--cc=Catalin.Marinas@arm.com \
--cc=Lorenzo.Pieralisi@arm.com \
--cc=Suzuki.Poulose@arm.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@kernel.org \
--cc=peterz@infradead.org \
--cc=preeti@linux.vnet.ibm.com \
--cc=rafael.j.wysocki@intel.com \
--cc=sudeep.holla@arm.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -360,14 +360,15 @@ void tick_broadcast_control(enum tick_br
if (!cpumask_test_and_set_cpu(cpu, tick_broadcast_mask)) {
/*
- * Only shutdown the cpu local device, if the
- * broadcast device exists and is in periodic
- * mode. The latter check prevents a hickup
- * during the switch from periodic to oneshot
- * mode.
+ * Only shutdown the cpu local device, if:
+ *
+ * - the broadcast device exists
+ * - the broadcast device is not a hrtimer based one
+ * - the broadcast device is in periodic mode to
+ * avoid a hickup during switch to oneshot mode
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®